@@ -11,12 +11,10 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public Lesser License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -117,6 +115,19 @@ static const FspError tag_get_list_translations [N_SPECIFIC_ERRORS] = {
   FSP_ERROR_UNKNOWN,                    /* 9 */
 };
 
+static const FspError set_license_translations [N_SPECIFIC_ERRORS] = {
+  FSP_ERROR_UNKNOWN,                    /* 0 */
+  FSP_ERROR_PHOTO_NOT_FOUND,            /* 1 */
+  FSP_ERROR_LICENSE_NOT_FOUND,          /* 2 */
+  FSP_ERROR_UNKNOWN,                    /* 3 */
+  FSP_ERROR_UNKNOWN,                    /* 4 */
+  FSP_ERROR_UNKNOWN,                    /* 5 */
+  FSP_ERROR_UNKNOWN,                    /* 6 */
+  FSP_ERROR_UNKNOWN,                    /* 7 */
+  FSP_ERROR_UNKNOWN,                    /* 8 */
+  FSP_ERROR_UNKNOWN,                    /* 9 */
+};
+
 static const FspError general_translations [N_GENERAL_ERRORS] = {
   FSP_ERROR_UNKNOWN,                    /* 10 */
   FSP_ERROR_UNKNOWN,                    /* 11 */
@@ -277,6 +288,10 @@ fsp_error_get_from_response_code        (FspErrorMethod method, gint code)
           retval = tag_get_list_translations[code];
           break;
 
+        case FSP_ERROR_METHOD_SET_LICENSE:
+          retval = set_license_translations[code];
+          break;
+
         default:
           retval = FSP_ERROR_UNKNOWN;
         }
