@@ -11,12 +11,10 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public Lesser License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -96,6 +94,9 @@ _added_to_group_parser                  (xmlDoc  *doc,
 static gpointer
 _get_tags_list_parser                   (xmlDoc  *doc,
                                          GError **error);
+static gpointer
+_set_license_parser                     (xmlDoc  *doc,
+                                         GError **error);
 
 static FspDataPhotoInfo *
 _get_photo_info_from_node               (xmlNode *node);
@@ -279,6 +280,8 @@ _get_error_method_from_parser           (gpointer (*body_parser)
     error_method = FSP_ERROR_METHOD_GROUP_ADD_PHOTO;
   else if (body_parser == _get_tags_list_parser)
     error_method = FSP_ERROR_METHOD_TAG_GET_LIST;
+  else if (body_parser == _set_license_parser)
+    error_method = FSP_ERROR_METHOD_SET_LICENSE;
 
   return error_method;
 }
@@ -846,6 +849,14 @@ _get_tags_list_parser                   (xmlDoc  *doc,
   return tagsList;
 }
 
+static gpointer
+_set_license_parser                     (xmlDoc  *doc,
+                                         GError **error)
+{
+  /* Dummy parser, as there is no response for this method */
+  return NULL;
+}
+
 static FspDataPhotoInfo *
 _get_photo_info_from_node               (xmlNode *node)
 {
@@ -1315,3 +1326,20 @@ fsp_parser_get_tags_list                (FspParser  *self,
   /* No return value for this method */
   return tags_list;
 }
+
+gpointer
+fsp_parser_set_license                  (FspParser  *self,
+                                         const gchar      *buffer,
+                                         gulong            buf_size,
+                                         GError          **error)
+{
+  g_return_val_if_fail (FSP_IS_PARSER (self), NULL);
+  g_return_val_if_fail (buffer != NULL, NULL);
+
+  /* Process the response */
+  _process_xml_response (self, buffer, buf_size,
+                         _set_license_parser, error);
+
+  /* No return value for this method */
+  return GINT_TO_POINTER ((gint)(*error == NULL));
+}
