@@ -11,12 +11,10 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public Lesser License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -135,6 +133,12 @@ fsp_parser_get_tags_list                (FspParser  *self,
                                          gulong            buf_size,
                                          GError          **error);
 
+gpointer
+fsp_parser_set_license                  (FspParser  *self,
+                                         const gchar      *buffer,
+                                         gulong            buf_size,
+                                         GError          **error);
+
 G_END_DECLS
 
 #endif
