@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -45,6 +43,8 @@ static const gchar *artists[] = {
 static const gchar *appdescr = N_("A Flickr remote organizer for GNOME\n");
 static const gchar *copyright = "(c) 2009-2011 Mario Sanchez Prada";
 static const gchar *website = "http://live.gnome.org/Frogr";
+
+#ifndef GTK_API_VERSION_3
 static const gchar *license =
   "frogr is free software: you can redistribute\n"
   "it and/or modify it under the terms of the GNU\n"
@@ -60,18 +60,18 @@ static const gchar *license =
   "You should have received a copy of the GNU General\n"
   "Public License along with frogr. If not, see\n"
   "http://www.gnu.org/licenses/\n\n"
-  "This program uses flickr API through the flicksoup\n"
-  "library (which comes bundled-in with frogr), but\n"
-  "it's neither approved nor certified by flickr.";
+  "This program uses its own implementation of Flickr API,\n"
+  "but it's neither approved nor certified by flickr.";
+#endif
 
-#if !GTK_CHECK_VERSION (2,23,0)
+#if !GTK_CHECK_VERSION (2,24,0)
 static void
 _frogr_about_dialog_uri_hook (GtkAboutDialog *about,
                               const gchar *link,
                               gpointer data)
 {
   gchar *uri = g_strconcat (data, link, NULL);
-  frogr_util_open_url_in_browser (uri);
+  frogr_util_open_uri (uri);
   g_free (uri);
 }
 #endif
@@ -88,7 +88,7 @@ frogr_about_dialog_show (GtkWindow *parent)
   logo = gdk_pixbuf_new_from_file (icon_full_path, NULL);
   g_free (icon_full_path);
 
-#if !GTK_CHECK_VERSION (2,23,0)
+#if !GTK_CHECK_VERSION (2,24,0)
   /* Install about dialog hooks */
   gtk_about_dialog_set_url_hook (_frogr_about_dialog_uri_hook, "", NULL);
   gtk_about_dialog_set_email_hook (_frogr_about_dialog_uri_hook, "mailto:",
@@ -104,7 +104,11 @@ frogr_about_dialog_show (GtkWindow *parent)
                          "artists", artists,
                          "comments", _(appdescr),
                          "copyright", copyright,
+#ifdef GTK_API_VERSION_3
+                         "license-type", GTK_LICENSE_GPL_3_0,
+#else
                          "license", license,
+#endif
                          "version", version,
                          "website", website,
                          "logo", logo,
