@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -35,11 +33,11 @@
 /* Path relative to the application data dir */
 #define MPICTURES_IMAGE "/images/mpictures.png"
 
-#define DIALOG_MIN_WIDTH 640
-#define DIALOG_MIN_HEIGHT 420
+#define DIALOG_MIN_WIDTH 740
+#define DIALOG_MIN_HEIGHT 400
 
-#define PICTURE_WIDTH 120
-#define PICTURE_HEIGHT 120
+#define PICTURE_WIDTH 180
+#define PICTURE_HEIGHT 180
 
 #define FROGR_DETAILS_DIALOG_GET_PRIVATE(object)                \
   (G_TYPE_INSTANCE_GET_PRIVATE ((object),                       \
@@ -57,6 +55,7 @@ typedef struct _FrogrDetailsDialogPrivate {
   GtkWidget *friend_cb;
   GtkWidget *family_cb;
   GtkWidget *show_in_search_cb;
+  GtkWidget *license_cb;
   GtkWidget *photo_content_rb;
   GtkWidget *sshot_content_rb;
   GtkWidget *other_content_rb;
@@ -66,10 +65,15 @@ typedef struct _FrogrDetailsDialogPrivate {
   GtkTextBuffer *text_buffer;
 
   GtkWidget *picture_img;
+  GtkWidget *picture_button;
+  GtkWidget *picture_container;
   GtkWidget *mpictures_label;
+  GdkPixbuf *mpictures_pixbuf;
 
   GtkTreeModel *treemodel;
   GSList *pictures;
+  gulong picture_button_handler_id;
+  gchar *reference_description;
 } FrogrDetailsDialogPrivate;
 
 /* Properties */
@@ -84,6 +88,18 @@ enum {
   N_COLS
 };
 
+static const gchar *license_descriptions[] = {
+  N_("Default (as specified in flickr)"),
+  N_("All rights reserved"),
+  N_("CC Attribution-NonCommercial-ShareAlike"),
+  N_("CC Attribution-NonCommercial"),
+  N_("CC Attribution-NonCommercial-NoDerivs"),
+  N_("CC Attribution"),
+  N_("CC Attribution-ShareAlike"),
+  N_("CC Attribution-NoDerivs"),
+  NULL
+};
+
 
 /* Prototypes */
 
@@ -99,7 +115,13 @@ static gboolean _completion_match_selected_cb (GtkEntryCompletion *widget, GtkTr
 
 static void _update_ui (FrogrDetailsDialog *self);
 
-static GdkPixbuf *_get_scaled_pixbuf (GdkPixbuf *pixbuf);
+static void _load_picture_from_disk (FrogrDetailsDialog *self);
+
+static void _load_picture_from_disk_cb (GObject *object,
+                                        GAsyncResult *res,
+                                        gpointer data);
+
+static void _place_picture_in_dialog_and_show (FrogrDetailsDialog *self);
 
 static void _fill_dialog_with_data (FrogrDetailsDialog *self);
 
@@ -109,6 +131,8 @@ static gboolean _save_data (FrogrDetailsDialog *self);
 
 static void _on_radio_button_clicked (GtkButton *tbutton, gpointer data);
 
+static void _on_picture_button_clicked (GtkButton *button, gpointer data);
+
 static void _on_toggle_button_toggled (GtkToggleButton *tbutton, gpointer data);
 
 static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data);
@@ -124,7 +148,6 @@ _create_widgets (FrogrDetailsDialog *self)
   GtkWidget *main_vbox = NULL;
   GtkWidget *vbox = NULL;
   GtkWidget *hbox = NULL;
-  GtkWidget *expander = NULL;
   GtkWidget *section_vbox = NULL;
   GtkWidget *internal_hbox = NULL;
   GtkWidget *visibility_vbox = NULL;
@@ -138,29 +161,49 @@ _create_widgets (FrogrDetailsDialog *self)
   GtkWidget *scroller = NULL;
   GtkTreeModel *model = NULL;
   gchar *markup = NULL;
+  gint i;
 
   priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
 
   main_vbox = gtk_dialog_get_content_area (GTK_DIALOG (self));
 
+#ifdef GTK_API_VERSION_3
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+#else
   hbox = gtk_hbox_new (FALSE, 0);
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_vbox_new (FALSE, 0);
+#endif
 
   /* Left side (image, radio buttons, checkboxes...) */
 
+  widget = gtk_button_new ();
+  gtk_widget_set_tooltip_text (widget, _("Open with image viewer"));
+  gtk_button_set_relief (GTK_BUTTON (widget), GTK_RELIEF_NONE);
+  priv->picture_button = widget;
+
+  align = gtk_alignment_new (0.5, 0, 0, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 0);
+  priv->picture_container = align;
+
   widget = gtk_image_new ();
   gtk_widget_set_size_request (widget, PICTURE_WIDTH, -1);
-  gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 6);
   priv->picture_img = widget;
 
   widget = gtk_label_new (NULL);
   gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
   priv->mpictures_label = widget;
+  priv->mpictures_pixbuf = NULL;
 
   /* Visibility */
 
+#ifdef GTK_API_VERSION_3
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  visibility_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
   section_vbox = gtk_vbox_new (FALSE, 6);
   visibility_vbox = gtk_vbox_new (FALSE, 6);
+#endif
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Visibility"));
@@ -172,7 +215,11 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
+#ifdef GTK_API_VERSION_3
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
   internal_hbox = gtk_hbox_new (FALSE, 6);
+#endif
 
   widget = gtk_radio_button_new_with_mnemonic (NULL, _("_Private"));
   gtk_box_pack_start (GTK_BOX (internal_hbox), widget, FALSE, FALSE, 0);
@@ -184,7 +231,11 @@ _create_widgets (FrogrDetailsDialog *self)
 
   gtk_box_pack_start (GTK_BOX (visibility_vbox), internal_hbox, FALSE, FALSE, 0);
 
+#ifdef GTK_API_VERSION_3
+  private_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
   private_vbox = gtk_vbox_new (FALSE, 6);
+#endif
 
   widget = gtk_check_button_new_with_mnemonic (_("Visible to _Family"));
   gtk_box_pack_start (GTK_BOX (private_vbox), widget, FALSE, FALSE, 0);
@@ -194,15 +245,30 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_box_pack_start (GTK_BOX (private_vbox), widget, FALSE, FALSE, 0);
   priv->friend_cb = widget;
 
+#ifdef GTK_API_VERSION_3
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
   internal_hbox = gtk_hbox_new (FALSE, 0);
+#endif
+
   gtk_box_pack_start (GTK_BOX (internal_hbox), private_vbox, FALSE, FALSE, 12);
   gtk_box_pack_start (GTK_BOX (visibility_vbox), internal_hbox, FALSE, FALSE, 0);
 
+#ifdef GTK_API_VERSION_3
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
   internal_hbox = gtk_hbox_new (FALSE, 0);
+#endif
+
   gtk_box_pack_start (GTK_BOX (internal_hbox), visibility_vbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (section_vbox), internal_hbox, FALSE, FALSE, 0);
 
+#ifdef GTK_API_VERSION_3
+  internal_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
   internal_hbox = gtk_hbox_new (FALSE, 0);
+#endif
+
   widget = gtk_check_button_new_with_mnemonic (_("_Show up in Global Search Results"));
   gtk_box_pack_start (GTK_BOX (internal_hbox), widget, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (section_vbox), internal_hbox, FALSE, FALSE, 0);
@@ -212,17 +278,22 @@ _create_widgets (FrogrDetailsDialog *self)
 
   /* Content type */
 
+#ifdef GTK_API_VERSION_3
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  content_type_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
   section_vbox = gtk_vbox_new (FALSE, 6);
   content_type_hbox = gtk_hbox_new (FALSE, 6);
+#endif
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Content type"));
-  expander = gtk_expander_new (markup);
-  gtk_expander_set_use_markup (GTK_EXPANDER (expander), TRUE);
+  widget = gtk_label_new (markup);
+  gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
   g_free (markup);
 
   align = gtk_alignment_new (0, 0, 0, 0);
-  gtk_container_add (GTK_CONTAINER (align), expander);
+  gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
   widget = gtk_radio_button_new_with_mnemonic (NULL, _("P_hoto"));
@@ -237,23 +308,28 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_box_pack_start (GTK_BOX (content_type_hbox), widget, FALSE, FALSE, 0);
   priv->other_content_rb = widget;
 
-  gtk_container_add (GTK_CONTAINER (expander), content_type_hbox);
+  gtk_box_pack_start (GTK_BOX (section_vbox), content_type_hbox, FALSE, FALSE, 0);
 
   gtk_box_pack_start (GTK_BOX (vbox), section_vbox, FALSE, FALSE, 6);
 
   /* Safety level */
 
+#ifdef GTK_API_VERSION_3
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  safety_level_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+#else
   section_vbox = gtk_vbox_new (FALSE, 6);
   safety_level_hbox = gtk_hbox_new (FALSE, 6);
+#endif
 
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
                                     _("Safety level"));
-  expander = gtk_expander_new (markup);
-  gtk_expander_set_use_markup (GTK_EXPANDER (expander), TRUE);
+  widget = gtk_label_new (markup);
+  gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
   g_free (markup);
 
   align = gtk_alignment_new (0, 0, 0, 0);
-  gtk_container_add (GTK_CONTAINER (align), expander);
+  gtk_container_add (GTK_CONTAINER (align), widget);
   gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
 
   widget = gtk_radio_button_new_with_mnemonic (NULL, _("S_afe"));
@@ -268,10 +344,42 @@ _create_widgets (FrogrDetailsDialog *self)
   gtk_box_pack_start (GTK_BOX (safety_level_hbox), widget, FALSE, FALSE, 0);
   priv->restricted_rb = widget;
 
-  gtk_container_add (GTK_CONTAINER (expander), safety_level_hbox);
+  gtk_box_pack_start (GTK_BOX (section_vbox), safety_level_hbox, FALSE, FALSE, 0);
 
   gtk_box_pack_start (GTK_BOX (vbox), section_vbox, FALSE, FALSE, 0);
 
+  /* License type */
+
+#ifdef GTK_API_VERSION_3
+  section_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
+  section_vbox = gtk_vbox_new (FALSE, 6);
+#endif
+
+  markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+                                    _("License type"));
+  widget = gtk_label_new (markup);
+  gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
+  g_free (markup);
+
+  align = gtk_alignment_new (0, 0, 0, 0);
+  gtk_container_add (GTK_CONTAINER (align), widget);
+  gtk_box_pack_start (GTK_BOX (section_vbox), align, FALSE, FALSE, 0);
+
+#if GTK_CHECK_VERSION (2,24,0)
+  widget = gtk_combo_box_text_new ();
+  for (i = 0; license_descriptions[i]; i++)
+    gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (widget), i, _(license_descriptions[i]));
+#else
+  widget = gtk_combo_box_new_text ();
+  for (i = 0; license_descriptions[i]; i++)
+    gtk_combo_box_insert_text (GTK_COMBO_BOX (widget), i, _(license_descriptions[i]));
+#endif
+  priv->license_cb = widget;
+  gtk_box_pack_start (GTK_BOX (section_vbox), widget, FALSE, FALSE, 0);
+
+  gtk_box_pack_start (GTK_BOX (vbox), section_vbox, FALSE, FALSE, 6);
+
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 6);
 
   /* Right side (text fields) */
@@ -330,7 +438,7 @@ _create_widgets (FrogrDetailsDialog *self)
 
   gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
 
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, TRUE, TRUE, 6);
 
   model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
   priv->treemodel = model;
@@ -516,35 +624,135 @@ _update_ui (FrogrDetailsDialog *self)
   gtk_widget_set_sensitive (priv->family_cb, !active);
 }
 
-static GdkPixbuf *
-_get_scaled_pixbuf (GdkPixbuf *pixbuf)
+static void
+_load_picture_from_disk (FrogrDetailsDialog *self)
 {
-  GdkPixbuf *scaled_pixbuf = NULL;
-  gint width;
-  gint height;
-  gint new_width;
-  gint new_height;
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
+  guint n_pictures;
 
-  /* Look for the right side to reduce */
-  width = gdk_pixbuf_get_width (pixbuf);
-  height = gdk_pixbuf_get_height (pixbuf);
-  if (width > height)
+  n_pictures = g_slist_length (priv->pictures);
+  if (n_pictures > 1)
     {
-      new_width = PICTURE_WIDTH;
-      new_height = (float)new_width * height / width;
+      gchar *mpictures_str = NULL;
+
+      /* Get the 'multiple pictures pixbuf' if not got yet (lazy approach) */
+      if (!priv->mpictures_pixbuf)
+        {
+          gchar *mpictures_full_path = NULL;
+
+          /* Set the image for editing multiple pictures */
+          mpictures_full_path = g_strdup_printf ("%s/" MPICTURES_IMAGE,
+                                                 frogr_util_get_app_data_dir ());
+          priv->mpictures_pixbuf = gdk_pixbuf_new_from_file (mpictures_full_path, NULL);
+          g_free (mpictures_full_path);
+        }
+
+      /* Just set the pixbuf in the image */
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), priv->mpictures_pixbuf);
+
+      /* Visually indicate how many pictures are being edited */
+      mpictures_str = g_strdup_printf (ngettext ("(%d Picture)", "(%d Pictures)", n_pictures), n_pictures);
+      gtk_label_set_text (GTK_LABEL (priv->mpictures_label), mpictures_str);
+      g_free (mpictures_str);
+
+      /* No need to spawn any async operation, show the dialog now */
+      _place_picture_in_dialog_and_show (self);
     }
   else
     {
-      new_height = PICTURE_HEIGHT;
-      new_width = (float)new_height * width / height;
+      FrogrPicture *picture = NULL;
+      gchar *file_uri = NULL;
+      GFile *gfile = NULL;
+
+      picture = FROGR_PICTURE (priv->pictures->data);
+      file_uri = (gchar *)frogr_picture_get_fileuri (picture);
+      gfile = g_file_new_for_uri (file_uri);
+
+      /* Asynchronously load the picture */
+      g_file_load_contents_async (gfile, NULL, _load_picture_from_disk_cb, self);
+    }
+}
+
+static void
+_load_picture_from_disk_cb (GObject *object,
+                            GAsyncResult *res,
+                            gpointer data)
+{
+  FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (data);
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
+  GFile *file = NULL;
+  GError *error = NULL;
+  gchar *contents = NULL;
+  gsize length = 0;
+
+  file = G_FILE (object);
+  if (g_file_load_contents_finish (file, res, &contents, &length, NULL, &error))
+    {
+      GdkPixbufLoader *pixbuf_loader = gdk_pixbuf_loader_new ();
+
+      if (gdk_pixbuf_loader_write (pixbuf_loader,
+                                   (const guchar *)contents,
+                                   length,
+                                   &error))
+        {
+          GdkPixbuf *pixbuf = NULL;
+          GdkPixbuf *s_pixbuf = NULL;
+
+          gdk_pixbuf_loader_close (pixbuf_loader, NULL);
+          pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
+
+          /* Get (scaled, and maybe rotated) pixbuf */
+          s_pixbuf = frogr_util_get_corrected_pixbuf (pixbuf, PICTURE_WIDTH, PICTURE_HEIGHT);
+
+          gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), s_pixbuf);
+          g_object_unref (s_pixbuf);
+
+          /* Everything should be fine by now, show it */
+          _place_picture_in_dialog_and_show (self);
+        }
+
+      g_object_unref (pixbuf_loader);
     }
 
-  /* Scale the pixbuf to its best size */
-  scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
-                                           new_width, new_height,
-                                           GDK_INTERP_TILES);
+  /* Show error to the user and finalize dialog if needed */
+  if (error)
+    {
+      GtkWindow *parent_window = NULL;
+      gchar *error_msg = NULL;
 
-  return scaled_pixbuf;
+      parent_window = gtk_window_get_transient_for (GTK_WINDOW (self));
+      gtk_widget_destroy (GTK_WIDGET (self));
+
+      if (error)
+        {
+          error_msg = g_strdup (error->message);
+          g_error_free (error);
+        }
+      else
+        error_msg = g_strdup (_("An error happened trying to load the picture"));
+
+      frogr_util_show_error_dialog (parent_window, error_msg);
+      g_free (error_msg);
+    }
+}
+
+static void
+_place_picture_in_dialog_and_show (FrogrDetailsDialog *self)
+{
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
+
+  gtk_button_set_image (GTK_BUTTON (priv->picture_button),
+                        priv->picture_img);
+
+  gtk_container_add (GTK_CONTAINER (priv->picture_container),
+                     priv->picture_button);
+
+  priv->picture_button_handler_id =
+    g_signal_connect (G_OBJECT (priv->picture_button), "clicked",
+                      G_CALLBACK (_on_picture_button_clicked),
+                      self);
+
+  gtk_widget_show_all (GTK_WIDGET (self));
 }
 
 static void
@@ -555,7 +763,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
 
   FrogrPicture *picture;
   GSList *item;
-  guint n_pictures;
+  GtkWidget *picture_widget;
   gchar *title_val = NULL;
   gchar *desc_val = NULL;
   gchar *tags_val = NULL;
@@ -563,6 +771,8 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   gboolean is_friend_val = FALSE;
   gboolean is_family_val = FALSE;
   gboolean show_in_search_val = FALSE;
+  gboolean license_inconsistent = FALSE;
+  FspLicense license_val = FSP_LICENSE_NONE;
   FspSafetyLevel safety_level_val = FSP_SAFETY_LEVEL_NONE;
   FspContentType content_type_val = FSP_CONTENT_TYPE_NONE;
 
@@ -577,6 +787,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   is_friend_val = frogr_picture_is_friend (picture);
   is_family_val = frogr_picture_is_family (picture);
   show_in_search_val = frogr_picture_show_in_search (picture);
+  license_val = frogr_picture_get_license (picture);
   safety_level_val = frogr_picture_get_safety_level (picture);
   content_type_val = frogr_picture_get_content_type (picture);
 
@@ -590,6 +801,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
       gboolean is_friend = FALSE;
       gboolean is_family = FALSE;
       gboolean show_in_search = FALSE;
+      FspLicense license = FSP_LICENSE_NONE;
       FspSafetyLevel safety_level = FSP_SAFETY_LEVEL_NONE;
       FspContentType content_type = FSP_CONTENT_TYPE_NONE;
 
@@ -610,6 +822,9 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
       is_family = frogr_picture_is_family (picture);
       show_in_search = frogr_picture_show_in_search (picture);
 
+      /* License */
+      license = frogr_picture_get_license (picture);
+
       /* Content type and safety level */
       safety_level = frogr_picture_get_safety_level (picture);
       content_type = frogr_picture_get_content_type (picture);
@@ -656,6 +871,12 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
           gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->show_in_search_cb),
                                               show_in_search_val != show_in_search);
         }
+
+      if (!license_inconsistent && license_val != license)
+        {
+          license_inconsistent = TRUE;
+        }
+
       if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->photo_content_rb)))
         {
           gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->photo_content_rb),
@@ -697,17 +918,27 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
       is_friend_val = is_friend;
       is_family_val = is_family;
       show_in_search_val = show_in_search;
+      license_val = license;
       content_type_val = content_type;
       safety_level_val = safety_level;
     }
 
   /* Fill in with data */
+
   if (title_val != NULL)
     gtk_entry_set_text (GTK_ENTRY (priv->title_entry), title_val);
 
   if (desc_val != NULL)
-    gtk_text_buffer_set_text (GTK_TEXT_BUFFER (priv->text_buffer),
-                              desc_val, -1);
+    {
+      gtk_text_buffer_set_text (GTK_TEXT_BUFFER (priv->text_buffer), desc_val, -1);
+      priv->reference_description = g_strstrip (g_strdup (desc_val));
+    }
+  else
+    {
+      /* We store "" in this case for ease further comparisons. */
+      priv->reference_description = g_strdup ("");
+    }
+
   if (tags_val != NULL)
     gtk_entry_set_text (GTK_ENTRY (priv->tags_entry), tags_val);
 
@@ -723,6 +954,11 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->show_in_search_cb),
                                 show_in_search_val);
 
+  if (license_inconsistent)
+    gtk_combo_box_set_active (GTK_COMBO_BOX (priv->license_cb), FSP_LICENSE_LAST + 1);
+  else
+    gtk_combo_box_set_active (GTK_COMBO_BOX (priv->license_cb), license_val + 1);
+
   if (content_type_val == FSP_CONTENT_TYPE_SCREENSHOT
       && !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->sshot_content_rb)))
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->sshot_content_rb), TRUE);
@@ -741,33 +977,17 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   else if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->safe_rb)))
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->safe_rb), TRUE);
 
-  n_pictures = g_slist_length (priv->pictures);
-  if (n_pictures > 1)
+  picture_widget = gtk_bin_get_child (GTK_BIN (priv->picture_container));
+  if (picture_widget)
     {
-      GdkPixbuf *pixbuf = NULL;
-      gchar *mpictures_str = NULL;
-      gchar *mpictures_full_path = NULL;
-
-      /* Set the image for editing multiple pictures */
-      mpictures_full_path = g_strdup_printf ("%s/" MPICTURES_IMAGE,
-					     frogr_util_get_app_data_dir ());
-      pixbuf = gdk_pixbuf_new_from_file (mpictures_full_path, NULL);
-      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), pixbuf);
-      g_object_unref (pixbuf);
-      g_free (mpictures_full_path);
-
-      /* Visually indicate how many pictures are being edited */
-      mpictures_str = g_strdup_printf (ngettext ("(%d Picture)", "(%d Pictures)", n_pictures), n_pictures);
-      gtk_label_set_text (GTK_LABEL (priv->mpictures_label), mpictures_str);
-      g_free (mpictures_str);
+      gtk_container_remove (GTK_CONTAINER (priv->picture_container),
+                            picture_widget);
     }
-  else
+
+  if (priv->picture_button_handler_id)
     {
-      /* Set pixbuf scaled to the right size */
-      GdkPixbuf *pixbuf = frogr_picture_get_pixbuf (picture);
-      GdkPixbuf *s_pixbuf = _get_scaled_pixbuf (pixbuf);
-      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), s_pixbuf);
-      g_object_unref (s_pixbuf);
+      g_signal_handler_disconnect (priv->picture_button,
+                                   priv->picture_button_handler_id);
     }
 
   /* Update UI */
@@ -776,6 +996,9 @@ _fill_dialog_with_data (FrogrDetailsDialog *self)
   /* Initial widget to grab focus */
   gtk_widget_grab_focus (priv->title_entry);
   gtk_editable_set_position (GTK_EDITABLE (priv->title_entry), -1);
+
+  /* Load the picture from disk, asynchronously */
+  _load_picture_from_disk (self);
 }
 
 static gboolean
@@ -817,6 +1040,7 @@ _save_data (FrogrDetailsDialog *self)
   gboolean is_friend;
   gboolean is_family;
   gboolean show_in_search;
+  FspLicense license;
   FspSafetyLevel safety_level;
   FspContentType content_type;
   gboolean result = FALSE;
@@ -854,6 +1078,10 @@ _save_data (FrogrDetailsDialog *self)
   show_in_search =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->show_in_search_cb));
 
+  /* License type (values in the combo are shifted +1) */
+  license = gtk_combo_box_get_active (GTK_COMBO_BOX (priv->license_cb)) - 1;
+
+  /* Content type */
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->photo_content_rb)))
     content_type = FSP_CONTENT_TYPE_PHOTO;
   else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->sshot_content_rb)))
@@ -861,6 +1089,7 @@ _save_data (FrogrDetailsDialog *self)
   else
     content_type = FSP_CONTENT_TYPE_OTHER;
 
+  /* Safety level */
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->safe_rb)))
     safety_level = FSP_SAFETY_LEVEL_SAFE;
   else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->moderate_rb)))
@@ -883,8 +1112,10 @@ _save_data (FrogrDetailsDialog *self)
 
           if (!g_str_equal (title, "") || (n_pictures <= 1))
             frogr_picture_set_title (picture, title);
-          if (!g_str_equal (description, "") || (n_pictures <= 1))
+
+          if (!g_str_equal (description, priv->reference_description))
             frogr_picture_set_description (picture, description);
+
           if (!g_str_equal (tags, "") || (n_pictures <= 1))
             frogr_picture_set_tags (picture, tags);
 
@@ -897,6 +1128,9 @@ _save_data (FrogrDetailsDialog *self)
           if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->show_in_search_cb)))
             frogr_picture_set_show_in_search (picture, show_in_search);
 
+          if (license >= FSP_LICENSE_NONE && license < FSP_LICENSE_LAST)
+            frogr_picture_set_license (picture, license);
+
           if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->photo_content_rb))
               && !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->sshot_content_rb))
               && !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->other_content_rb)))
@@ -960,7 +1194,32 @@ _on_toggle_button_toggled (GtkToggleButton *tbutton, gpointer data)
   _update_ui (self);
 }
 
-static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
+static void
+_on_picture_button_clicked (GtkButton *button, gpointer data)
+{
+  FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (data);
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
+  GSList *current_pic = NULL;
+  GList *uris_list = NULL;
+  FrogrPicture *picture = NULL;
+  gchar *fileuri = NULL;
+
+  for (current_pic = priv->pictures; current_pic; current_pic = g_slist_next (current_pic))
+    {
+      picture = FROGR_PICTURE (current_pic->data);
+      fileuri = g_strdup (frogr_picture_get_fileuri (picture));
+
+      /* Dupped uris in the GList must NOT be freed here */
+      uris_list = g_list_append (uris_list, fileuri);
+    }
+
+  frogr_util_open_images_in_viewer (uris_list);
+
+  g_list_free (uris_list);
+}
+
+static void
+_dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
 {
   FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (dialog);
 
@@ -1018,6 +1277,12 @@ _frogr_details_dialog_dispose (GObject *object)
 {
   FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (object);
 
+  if (priv->mpictures_pixbuf)
+    {
+      g_object_unref (priv->mpictures_pixbuf);
+      priv->mpictures_pixbuf = NULL;
+    }
+
   if (priv->pictures)
     {
       g_slist_foreach (priv->pictures, (GFunc)g_object_unref, NULL);
@@ -1034,6 +1299,16 @@ _frogr_details_dialog_dispose (GObject *object)
   G_OBJECT_CLASS(frogr_details_dialog_parent_class)->dispose (object);
 }
 
+static void
+_frogr_details_dialog_finalize (GObject *object)
+{
+  FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (object);
+
+  g_free (priv->reference_description);
+
+  G_OBJECT_CLASS(frogr_details_dialog_parent_class)->finalize (object);
+}
+
 static void
 frogr_details_dialog_class_init (FrogrDetailsDialogClass *klass)
 {
@@ -1044,6 +1319,7 @@ frogr_details_dialog_class_init (FrogrDetailsDialogClass *klass)
   obj_class->set_property = _frogr_details_dialog_set_property;
   obj_class->get_property = _frogr_details_dialog_get_property;
   obj_class->dispose = _frogr_details_dialog_dispose;
+  obj_class->finalize = _frogr_details_dialog_finalize;
 
   /* Install properties */
   pspec = g_param_spec_pointer ("pictures",
@@ -1065,6 +1341,8 @@ frogr_details_dialog_init (FrogrDetailsDialog *self)
 
   priv->treemodel = NULL;
   priv->pictures = NULL;
+  priv->picture_button_handler_id = 0;
+  priv->reference_description = NULL;
 
   _create_widgets (self);
 
@@ -1104,6 +1382,4 @@ frogr_details_dialog_show (GtkWindow *parent, GSList *fpictures, GSList *tags)
 
   _fill_dialog_with_data (self);
   _populate_treemodel_with_tags (self, tags);
-
-  gtk_widget_show_all (GTK_WIDGET (self));
 }
