@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -29,6 +27,18 @@
 #define APP_SHORTNAME PACKAGE
 #define APP_VERSION VERSION
 
+/* Max width and heigth for pictures in the icon view.
+ *
+ * These values will impact directly memory comsumption because it
+ * will be the size of the pixbuf that will remain in memory during
+ * the whole life of every FrogrPicture object.
+ */
+#define IV_THUMB_WIDTH 136
+#define IV_THUMB_HEIGHT 136
+
+/* Padding to be used in the icon view */
+#define IV_THUMB_PADDING 4
+
 #if DEBUG_ENABLED
 #define DEBUG(...) g_debug (__VA_ARGS__);
 #else
