@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -100,18 +98,23 @@ _compare_pictures_by_property (FrogrPicture *p1, FrogrPicture *p2,
     result = g_value_get_long (&value1) - g_value_get_long (&value2);
   else if (G_VALUE_HOLDS_STRING (&value1))
     {
-      gchar *str1 = NULL;
-      gchar *str2 = NULL;
+      const gchar *str1 = NULL;
+      const gchar *str2 = NULL;
+      gchar *str1_cf = NULL;
+      gchar *str2_cf = NULL;
 
       /* Comparison of strings require some additional work to take
          into account the different rules for each locale */
-      str1 = g_utf8_casefold (g_value_get_string (&value1), -1);
-      str2 = g_utf8_casefold (g_value_get_string (&value2), -1);
+      str1 = g_value_get_string (&value1);
+      str2 = g_value_get_string (&value2);
 
-      result = g_utf8_collate (str1, str2);
+      str1_cf = g_utf8_casefold (str1 ? str1 : "", -1);
+      str2_cf = g_utf8_casefold (str2 ? str2 : "", -1);
 
-      g_free (str1);
-      g_free (str2);
+      result = g_utf8_collate (str1_cf, str2_cf);
+
+      g_free (str1_cf);
+      g_free (str2_cf);
     }
   else
     g_warning ("Unsupported type for property used for sorting");
