@@ -375,6 +375,8 @@ _on_icon_view_drag_data_received (GtkWidget *widget,
       gchar *fileuri = g_strdup (fileuris_array[i]);
       if (fileuri && !g_str_equal (g_strstrip (fileuri), ""))
         fileuris_list = g_slist_append (fileuris_list, fileuri);
+      else
+        g_free (fileuri);
     }
 
   /* Load pictures */
@@ -686,6 +688,7 @@ _on_icon_view_query_tooltip (GtkWidget *icon_view,
       /* Free memory */
       gtk_tree_path_free (path);
       g_free (tooltip_str);
+      g_free (filesize);
       g_free (filesize_str);
       g_free (filesize_markup);
       g_free (datetime_markup);
@@ -785,6 +788,7 @@ _add_picture_to_ui (FrogrMainView *self, FrogrPicture *picture)
                       -1);
 
   g_object_ref (picture);
+  g_object_unref (pixbuf);
 
   /* Reorder if needed */
   if (priv->sorting_criteria != SORT_AS_LOADED || priv->sorting_reversed)
@@ -1238,6 +1242,7 @@ _update_account_menu_items (FrogrMainView *mainview)
 
           gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), value);
         }
+      g_list_free (all_items);
     }
 }
 
@@ -1500,6 +1505,7 @@ _frogr_main_view_finalize (GObject *object)
 
   gtk_widget_destroy (priv->pictures_ctxt_menu);
   gtk_widget_destroy (GTK_WIDGET (priv->window));
+  g_object_unref (G_OBJECT (priv->builder));
 
   G_OBJECT_CLASS(frogr_main_view_parent_class)->finalize (object);
 }
@@ -1530,7 +1536,7 @@ frogr_main_view_init (FrogrMainView *self)
   GtkWidget *progress_label;
   const gchar *icons_path;
   gchar *full_path;
-  GList *icons;
+  GList *icons = NULL;
 
 #ifdef MAC_INTEGRATION
   GtkOSXApplication *osx_app;
@@ -1549,27 +1555,27 @@ frogr_main_view_init (FrogrMainView *self)
   /* Provide a default icon list in several sizes */
   icons_path = frogr_util_get_icons_dir ();
   full_path = g_strdup_printf ("%s/" MAIN_VIEW_ICON("128x128"), icons_path);
-  icons = g_list_prepend (NULL, gdk_pixbuf_new_from_file (full_path, NULL));
+  icons = g_list_prepend (icons, gdk_pixbuf_new_from_file (full_path, NULL));
   g_free (full_path);
 
   full_path = g_strdup_printf ("%s/" MAIN_VIEW_ICON("64x64"), icons_path);
-  icons = g_list_prepend (NULL, gdk_pixbuf_new_from_file (full_path, NULL));
+  icons = g_list_prepend (icons, gdk_pixbuf_new_from_file (full_path, NULL));
   g_free (full_path);
 
   full_path = g_strdup_printf ("%s/" MAIN_VIEW_ICON("48x48"), icons_path);
-  icons = g_list_prepend (NULL, gdk_pixbuf_new_from_file (full_path, NULL));
+  icons = g_list_prepend (icons, gdk_pixbuf_new_from_file (full_path, NULL));
   g_free (full_path);
 
   full_path = g_strdup_printf ("%s/" MAIN_VIEW_ICON("32x32"), icons_path);
-  icons = g_list_prepend (NULL, gdk_pixbuf_new_from_file (full_path, NULL));
+  icons = g_list_prepend (icons, gdk_pixbuf_new_from_file (full_path, NULL));
   g_free (full_path);
 
   full_path = g_strdup_printf ("%s/" MAIN_VIEW_ICON("24x24"), icons_path);
-  icons = g_list_prepend (NULL, gdk_pixbuf_new_from_file (full_path, NULL));
+  icons = g_list_prepend (icons, gdk_pixbuf_new_from_file (full_path, NULL));
   g_free (full_path);
 
   full_path = g_strdup_printf ("%s/" MAIN_VIEW_ICON("16x16"), icons_path);
-  icons = g_list_prepend (NULL, gdk_pixbuf_new_from_file (full_path, NULL));
+  icons = g_list_prepend (icons, gdk_pixbuf_new_from_file (full_path, NULL));
   g_free (full_path);
 
   gtk_window_set_default_icon_list (icons);
